<?php
// filesystem.inc.php

class FSnode{
        var $fsparent;
        var $children;
        var $type;
        var $name;
        var $id;

        function FSnode($n, $t, $p = false){
                $this->name = $n;
                $this->type = $t;  //f(ile), d(irectory), l(ink)
                $this->parent = &$p; //if false, this is root
                $this->children = array();
                $this->id = false;
        }

        function mdir($dirname){
                $d = new FSnode($dirname, 'd', $this);
                $this->children[$dirname] = &$d;
                return $d;
        }

        function mfiles($fdata){
                foreach($fdata as $filename => $fileid){
                        $f = new FSnode($filename,'f',$this);
                        $f->id = $fileid;
                        $this->children[$filename] = $f;
                }
        }

        function mlink($linkname, $target){
                $l = new FSnode($linkname, 'l', $this);
                $l->id = $target;
                $this->children[$linkname] = $l;
        }

        function abspath(){
                $i = $this;
                $p = array();
                $path = '';
                do{
                        $path = $i->name.'/'.$path;
                        $i = $i->parent;
                }while($i);
                return $path;
        }

        function getlink(){
                switch($this->type){
                        case 'f':
                        case 'l':
                                return "showpost(".$this->id.");";
                        case 'd':
                                return "ls('".addslashes(safe_escape($this->abspath()))."');";
                        default:
                                return false;
                }
        }
}

function initFS(){
        $fsroot=new FSnode('','d');
//5     $authsnode = $fsroot->mdir('authors');
/* 4 */
        $authsnode=new FSnode('authors', 'd', &$fsroot);
        $fsroot->children['authors']=&$authsnode;
/* /4 */
        $al = authorList();
        foreach($al as $author){
//5             $authnode = $authsnode->mdir($author);
/* 4 */
		$authnode=new FSnode($author, 'd', &$authsnode);
                $authnode->mfiles(postsByAuthor($author));
        	$authsnode->children[$author]=$authnode;
/* /4 */
        }
        $catsnode = $fsroot;
        if(get_option('cli_categories_in_root') == 'categories'){
//5             $catsnode = $fsroot->mdir('categories');
/* 4 */
                $catsnode = new FSnode('categories','d', &$fsroot);
                $fsroot->children['categories'] = &$catsnode;
/* /4 */
        }
        if(get_option('cli_categories_as_tree') == 'tree'){
           insertCategories(&$catsnode, 0);
        }else{ // categories in flat list
                if(function_exists('get_categories')){
                        $cl = get_categories();
                }else{
                        $cl = get_the_category();
                }
                foreach($cl as $category){
//5                     $catnode = $catsnode->mdir($category->cat_name);
/* 4 */                 $catnode = new FSnode($category->cat_name,'d',$catsnode);
                        $catnode->mfiles(postsInCategory($category->cat_ID));
                        $catsnode->children[$category->cat_name] = $catnode;
/* /4 */
                }
        }
        $fsroot->mfiles(pageList());
        $fsroot->mlink('latest', "'latest'");
        return $fsroot;
}

function insertCategories($filenode, $fsparent){
       	if(function_exists('get_categories')){
               	$cats = get_categories('child_of='.$fsparent);
	}else{
		$cats = get_the_category();
	}
        foreach($cats as $c){
                if($fsparent != $c->category_parent) continue; //you'd think I wouldn't have to check this, but I do. Crazy! The bug is in get_categories.
//5             $catnode = $filenode->mdir($c->cat_name);
/* 4 */         $catnode = new FSnode($c->cat_name,'d',$filenode);
                $catnode->mfiles(postsInCategory($c->cat_ID));
                $filenode->children[$c->cat_name]=$catnode;
/* /4 */
                insertCategories(&$catnode, $c->cat_ID);
        }
}

function absolute($path){
        if($path{0} != '/'){
                $path='/'.$_SESSION['path'].'/'.$path;
        }
        $path = str_replace('//','/', $path);
        return $path;
}

function path_expand($p){
        global $username;
        if($p{0} == '~'){
                if($p == '~' || $p{1} == '/'){
                        $p = str_replace('~', '/authors/'.$username, $p);
                }else{
                        $p = str_replace('~', '/authors/', $p);
                }
        }
        $p = absolute($p);
        $p = explode('/',$p);

        $newpath = array();
        foreach($p as $c){
                if($c == '..' && count($newpath > 0)){
                        array_pop($newpath);
                }elseif($c == '.'){
                        continue;
                }elseif($c != ''){
                        array_push($newpath, $c);
                }
        }
        $p = '/'.implode('/', $newpath);
        return $p;
}

function path_exists($path){
        global $root;
        $path = absolute($path);
        $path = explode('/', $path);
        $d=$root;
        for($i = 0; $i < count($path); $i++){
                if($path[$i] != ''){
                        $hmm = @$d->children[$path[$i]];
                        if($hmm && $hmm->type == 'd'){
                                $d = $hmm;
                        }else{
                                $d=false;
                                continue;
                        }
                }
        }
        if(!$d) return false;
        return $d;
}

function fs_tab_completions($t){
        $t = path_expand($t);
        $d = dirname($t);
        $b = basename($t);
        if($filenode = path_exists($t)){
                $b='';
        }elseif(!$filenode = path_exists($d)){
                return false;
        }
        $candidates = array_keys($filenode->children);
        $result = array();
        foreach($candidates as $c){
                if($b == '' || strpos($c, $b) === 0){
                        if($filenode->children[$c]->type == 'd') $c.='/';
                        $result[] = $c;
                }
        }
        if($result)
                return array_unique($result);
        return $result;
}

/* these should all return array(name => js link, ...) */

function authorIDs(){ // returns array of all (author) users
        global $wpdb;
        $authors = array();
        $authors = $wpdb->get_col("SELECT DISTINCT post_author FROM $wpdb->posts");
        ksort($authors);
        return $authors;
}

function authorList(){
        $aids = authorIDs();
        $a = array();
        foreach($aids as $id){
                $u = get_userdata($id);
                $a[] = $u->display_name;
        }
        sort($a);
        return $a;
}

function pageList(){ // list of all 'pages' (not posts) [+ blogroll?]
        global $wpdb;
        $q = "SELECT `ID` FROM $wpdb->posts
                WHERE `post_type`='page' AND `post_status`='publish'";
        $pageIDs = $wpdb->get_col($q);
        $pages = array();
        if($pageIDs){
	        foreach($pageIDs as $id){
	                $post = get_post($id);
                	$pages[$post->post_title] = $id;
	        }
        }
        return $pages;
}

function postsByAuthor($a){ // author nicename
        global $wpdb;
        $result = array();
        $q = "SELECT p.`post_title` as t, p.`ID` as i
                FROM $wpdb->posts as p, $wpdb->users as u
                WHERE `post_status`='publish'
                AND u.ID=p.post_author
                AND u.display_name='$a'";
        $r = $wpdb->query($q);
        for($i = 0; $i < $r; $i++){
                $row = $wpdb->get_row(null, ARRAY_A, $i);
                $result[$row['t']] = $row['i'];
        }
        return $result;
}

function postsInCategory($cid){ // category name
        global $wpdb;
        $result = array();
        $q = "SELECT p.`post_title` as t, p.`ID` as i
                FROM $wpdb->posts as p,
                         $wpdb->categories as c,
                         $wpdb->post2cat as p2
                WHERE p.`post_status`='publish'
                AND p2.post_ID=p.ID
                AND c.cat_ID='$cid'
                AND c.cat_ID=p2.category_id";
        $r=$wpdb->query($q);
        for($i = 0; $i < $r; $i++){(
                $row = $wpdb->get_row(null, ARRAY_A, $i));
                $result[$row['t']] = $row['i'];
        }
        return $result;
}

function traverse($d, $depth = 0){ // for recursive listing
        for($i = 0; $i < $depth; $i++){
                e('&nbsp;&nbsp;&nbsp;&nbsp;');
        }
        e('<span class="linky" onclick="'.$d->getlink().'">'.$d->name.'/:</span>');
        e('<br />');
        foreach($d->children as $dn){
                if($dn->type == 'd'){
                        traverse($dn, $depth + 1);
                }else{
                        for($i = 0;$i <= $depth; $i++){
                                e('&nbsp;&nbsp;&nbsp;&nbsp;');
                        }
                        e('<span class="linky" onclick="'.$dn->getlink().'">'.$dn->name);
                        if($dn->type == 'l')e('@');
                        e('</span><br />');
                }
        }
}

$root=initFS();
?>
